@@ -20,6 +20,11 @@
 # See http://www.gnu.org/licenses/ for more information.
 
 import sys
+
+import sip
+sip.setapi("QString", 2)
+sip.setapi("QVariant", 2)
+
 from PyKDE4.kdecore import (ki18n, ki18nc,
     KAboutData, KCmdLineArgs, KCmdLineOptions, KComponentData, KLocalizedString)
 
@@ -45,7 +50,7 @@ aboutData.setTranslator(
     ki18nc("NAME OF TRANSLATORS", "Your name"),
     ki18nc("EMAIL OF TRANSLATORS", "i18n@frescobaldi.org"))
  
-KCmdLineArgs.init(sys.argv, aboutData)
+KCmdLineArgs.init(list(sys.argv), aboutData)
 KComponentData(aboutData).dirs().addPrefix("@CMAKE_INSTALL_PREFIX@")
 
 options = KCmdLineOptions()
@@ -61,13 +66,17 @@ KCmdLineArgs.addCmdLineOptions(options)
 args = KCmdLineArgs.parsedArgs()
 
 app = not args.isSet("new") and runningApp() or newApp()
+if args.isSet("start"):
+    app.startSession(args.getOption("start"))
 docs = [app.openUrl(args.url(c), args.getOption("encoding"))
         for c in range(args.count())]
 if docs:
     docs[-1].setActive()
-    if args.isSet("line") or args.isSet("column"):
-        line = int(args.getOption("line") or 1)
-        col = int(args.getOption("column") or 0)
-        docs[-1].setCursorPosition(line, col, args.isSet("smart"))
+    line = args.getOption("line")
+    if line.isdigit():
+        line = int(line) or 1
+        column = args.getOption("column")
+        column = int(column) if column.isdigit() else 0
+        docs[-1].setCursorPosition(line, column, args.isSet("smart"))
     
 app.run()
