@@ -32,11 +32,11 @@ from PyKDE4.kdecore import KGlobal, i18n
 from PyKDE4.kdeui import KDialog, KIcon, KMessageBox
 from PyKDE4.ktexteditor import KTextEditor
 
-import ly.rx, ly.pitch, ly.parse, ly.tokenize, ly.tools, ly.version
+import ly.rx, ly.dynamic, ly.pitch, ly.parse, ly.tokenize, ly.tools, ly.version
 from kateshell.app import cacheresult
 from kateshell.widgets import promptText
 from kateshell.mainwindow import addAccelerators
-from frescobaldi_app.mainapp import lilyPondCommand
+from frescobaldi_app.mainapp import lilyPondCommand, lilyPondVersion
 
 
 class DocumentManipulator(object):
@@ -61,7 +61,11 @@ class DocumentManipulator(object):
     def changeLanguage(self, lang):
         """
         Change the LilyPond pitch name language in our document to lang.
+        
+        This is a bit hairy because LilyPond has a new syntax for this since 2.14,
+        \language "name", while the old \include "name.ly" still is supported as well.
         """
+        newSyntax = (self.doc.lilyPondVersion() or lilyPondVersion()) >= (2, 13, 38)
         text, start = self.doc.selectionOrDocument()
         try:
             changes, includeCommandChanged = ly.tools.translate(text, lang, start)
@@ -77,13 +81,21 @@ class DocumentManipulator(object):
         with self.doc.editContext():
             changes.applyToCursor(EditCursor(self.doc.doc))
             if not start and not includeCommandChanged:
-                self.addLineToTop('\\include "{0}.ly"'.format(lang))
+                if newSyntax:
+                    self.addLineToTop('\\language "{0}"'.format(lang))
+                else:
+                    self.addLineToTop('\\include "{0}.ly"'.format(lang))
         if start and not includeCommandChanged:
             KMessageBox.information(self.doc.app.mainwin,
-                '<p>{0}</p><p><tt>\\include "{1}.ly"</tt></p>'.format(
-                i18n("The pitch language of the selected text has been "
-                     "updated, but you need to manually add the following "
-                     "command to your document:"), lang),
+                '<p>{0}</p>'
+                '<p><tt>\\include "{1}.ly"</tt> {2}</p>'
+                '<p><tt>\\language "{1}"</tt> {3}</p>'.format(
+                    i18n("The pitch language of the selected text has been "
+                         "updated, but you need to manually add the following "
+                         "command to your document:"),
+                    lang,
+                    i18n("(for LilyPond below 2.14), or"),
+                    i18n("(for LilyPond 2.14 and higher.)")),
                 i18n("Pitch Name Language"))
 
     def addLineToTop(self, text):
@@ -527,7 +539,58 @@ class DocumentManipulator(object):
         """
         Insert a \\bar ".." command with the given type.
         """
-        self.doc.view.insertText('\\bar "{0}"'.format(bar))
+        self.insertIndented('\\bar "{0}"'.format(bar))
+    
+    def insertBreathingSign(self, sign):
+        """
+        Insert a \\breathe mark with possibly an override for another shape.
+        """
+        if sign == 'rcomma':
+            text = '\\breathe'
+        else:
+            text = ("\\once \\override BreathingSign #'text = "
+                    '#(make-musicglyph-markup "scripts.{0}")\n'
+                    "\\breathe").format(sign.replace('_', '.'))
+        self.insertIndented(text)
+        
+    def insertIndented(self, text, cursor=None, wholeLines=None):
+        """Inserts text on the given or current cursor position.
+        
+        The following protocol is used:
+        
+        - if there is a newline in the text or wholeLines is True:
+            add newlines if necessary to have the text on its own lines.
+        - if wholeLines is False:
+            don't add newlines
+        - if no newlines were added:
+            - if the cursor is at a non-space character:
+                add a space after the text
+            - if the cursor is just after a non-space character:
+                add a space before the text.
+        """
+        cursor = cursor or self.doc.view.cursorPosition()
+        line, col = cursor.line(), cursor.column()
+        before = self.doc.line(line)[:col]
+        after = self.doc.line(line)[col:]
+        remove = None
+        if wholeLines or (wholeLines is None and '\n' in text):
+            if not isblank(after):
+                text += '\n'
+                spaces = len(after) - len(after.lstrip())
+                if spaces:
+                    remove = KTextEditor.Range(line, col, line, col + spaces)
+            text = '\n' + self.doc.indent(text, self.doc.currentIndent(cursor))
+            if isblank(before):
+                text = text.lstrip()
+        else:
+            if before and not before[-1].isspace():
+                text = ' ' + text
+            if after and not after[0].isspace():
+                text += ' '
+        if remove:
+            self.doc.doc.replaceText(remove, text)
+        else:
+            self.doc.doc.insertText(cursor, text)
         
     def insertTemplate(self, text, cursor=None, remove=None, doIndent=True):
         """
@@ -570,6 +633,7 @@ class DocumentManipulator(object):
     def addArticulation(self, art):
         """
         Add artication to selected notes or chord, or just insert it.
+        See lqi.py Articulations.
         """
         text = self.doc.selectionText()
         if text:
@@ -589,6 +653,164 @@ class DocumentManipulator(object):
         else:
             self.adjustCursorToChords()
             self.doc.view.insertText(art)
+    
+    _spannerDynamics = {
+        'hairpin_cresc': '\\<',
+        'hairpin_dim':   '\\>',
+        'cresc':         '\\cresc',
+        'decresc':       '\\decresc',
+        'dim':           '\\dim',
+    }
+        
+    def addDynamic(self, name, direction):
+        """
+        Add dynamics with name, direction (-1, 0 or 1).
+        See lqi.py Dynamics.
+        """
+        isSpanner = name in self._spannerDynamics
+        if isSpanner:
+            dynamic = self._spannerDynamics[name]
+        elif name in ly.dynamic.marks:
+            dynamic = '\\' + name
+        else:
+            return
+        direction = ['_', '', '^'][direction+1]
+        
+        text = self.doc.selectionText()
+        if text:
+            items = list(m for m in ly.rx.chord_rest.finditer(text) if m.group('full'))
+            if len(items) >= 2:
+                # match objects for the first note and the last in the selection
+                first, last = items[0], items[-1]
+                
+                # text after the first note/chord/rest
+                afterFirst = text[first.end('full'):]
+                
+                # text after the last note/chord/rest
+                afterLast = text[last.end('full'):]
+                selRange = self.doc.view.selectionRange() # copy othw. crash in KDE 4.3 /PyQt 4.5.x.
+                docRange = self.doc.doc.documentRange()
+                afterLast += self.doc.doc.text(
+                    KTextEditor.Range(selRange.end(), docRange.end()))[:10]
+                
+                start = Cursor(selRange.start())
+                start.walk(text[:first.end('full')])
+                end = Cursor(selRange.start())
+                end.walk(text[:last.end('full')])
+                
+                if isSpanner:
+                    with self.doc.editContext():
+                        # don't terminate the spanner if it already ends with a dynamic
+                        if not ly.rx.dynamic_mark.match(afterLast):
+                            self.doc.doc.insertText(end.kteCursor(), '\\!')
+                        # skip a dynamic mark that might already be at the start
+                        m = ly.rx.dynamic_mark.match(afterFirst)
+                        if m:
+                            start.walk(text[:m.end()])
+                            direction = ''
+                        # on this place, insert the spanner
+                        self.doc.doc.insertText(start.kteCursor(), direction + dynamic)
+                else:
+                    # if a spanner ends on the end, replace it with our dynamic
+                    if afterLast.startswith('\\!'):
+                        r = KTextEditor.Range(
+                            end.line, end.column, end.line, end.column + 2)
+                        self.doc.doc.replaceText(r, dynamic)
+                    elif ly.rx.dynamic_mark.match(afterFirst):
+                        self.doc.doc.insertText(end.kteCursor(), direction + dynamic)
+                    else:
+                        self.doc.doc.insertText(start.kteCursor(), direction + dynamic)
+                return
+        self.adjustCursorToChords()
+        self.doc.view.insertText(direction + dynamic)
+        
+    def addSpanner(self, name, direction=0):
+        """ Add a simple spanner to the selected music. """
+        if name == "slur":
+            spanner = '(', ')'
+        if name == "phrasing_slur":
+            spanner = '\\(', '\\)'
+        elif name == "beam":
+            spanner = '[', ']'
+        elif name == "trill":
+            spanner = '\\startTrillSpan', '\\stopTrillSpan'
+        direction = ['_', '', '^'][direction+1]
+        
+        text = self.doc.selectionText()
+        if not text:
+            return
+            
+        items = list(m for m in ly.rx.chord_rest.finditer(text) if m.group('full'))
+        if len(items) < 2:
+            return # can't add spanner to one note or chord
+        
+        first, last = items[0], items[-1]
+        selRange = self.doc.view.selectionRange() # copy othw. crash in KDE 4.3 /PyQt 4.5.x.
+        start = Cursor(selRange.start())
+        start.walk(text[:first.end('full')])
+        end = Cursor(selRange.start())
+        end.walk(text[:last.end('full')])
+        
+        with self.doc.editContext():
+            self.doc.doc.insertText(end.kteCursor(), spanner[1])
+            self.doc.doc.insertText(start.kteCursor(), direction + spanner[0])
+    
+    _arpeggioTypes = {
+        'arpeggio_normal': '\\arpeggioNormal',
+        'arpeggio_arrow_up': '\\arpeggioArrowUp',
+        'arpeggio_arrow_down': '\\arpeggioArrowDown',
+        'arpeggio_bracket': '\\arpeggioBracket',
+        'arpeggio_parenthesis': '\\arpeggioParenthesis',
+    }
+                
+    def addArpeggio(self, name):
+        """ Ad an arpeggio to the current chord. """
+        try:
+            arpeggioType = self._arpeggioTypes[name]
+        except KeyError:
+            return
+        
+        self.adjustCursorToChords()
+        lastUsed = '\\arpeggioNormal'
+        tokenizer = ly.tokenize.Tokenizer()
+        for token in tokenizer.tokens(self.doc.textToCursor()):
+            if (isinstance(token, tokenizer.Command)
+                    and token in self._arpeggioTypes.values()):
+                lastUsed = token
+            
+        with self.doc.editContext():
+            self.doc.view.insertText('\\arpeggio')
+            if arpeggioType != lastUsed:
+                indent = re.match(r'\s*', self.doc.line()).group()
+                cursor = self.doc.view.cursorPosition()
+                cursor.setColumn(len(indent))
+                self.doc.doc.insertText(cursor, arpeggioType + '\n' + indent)
+    
+    _glissandoStyles = {
+        'glissando_normal': '',
+        'glissando_dashed': 'dashed-line',
+        'glissando_dotted': 'dotted-line',
+        'glissando_zigzag': 'zigzag',
+        'glissando_trill':  'trill',
+    }
+    
+    def addGlissando(self, name):
+        """ Add a Glissando to the current chord.
+        
+        Currently uses \\tweak to set different styles.
+        
+        """
+        try:
+            style = self._glissandoStyles[name]
+        except KeyError:
+            return
+            
+        self.adjustCursorToChords()
+        if not style:
+            self.doc.view.insertText("\\glissando")
+        else:
+            self.doc.view.insertText(
+                "-\\tweak #'style #'{0} \\glissando".format(style))
         
     def wrapSelection(self, text, before='{', after='}', alwaysMultiLine=False):
         """
