@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 """ Hyphenator functionality for Frescobaldi """
 
 import locale, os, re
@@ -25,17 +27,16 @@ from glob import glob
 import ly.rx
 from hyphenator import Hyphenator
 
-from PyQt4.QtCore import QVariant
-from PyQt4.QtGui import QLabel, QListWidget
+from PyQt4.QtGui import QLabel, QListWidget, QVBoxLayout
 from PyKDE4.kdecore import KConfig, KGlobal, i18n
-from PyKDE4.kdeui import KDialog, KVBox
+from PyKDE4.kdeui import KDialog, KMessageBox
 
 try:
     language, encoding = locale.getdefaultlocale()
 except ValueError:
     language, encoding = None, None
 
-defaultPaths = (
+defaultPaths = [
     'share/dict/ooo',
     'lib/openoffice/share/dict/ooo',
     '/opt/OpenOffice.org/share/dict/ooo',
@@ -47,7 +48,7 @@ defaultPaths = (
     'share/myspell',
     'share/myspell/dicts',
     'share/hunspell',
-)
+]
 
 hyphdicts = {}
 
@@ -60,23 +61,22 @@ def findDicts():
     def paths():
         """ build a list of paths based on config """
         # in which prefixes to look for relative paths
-        prefixes = unicode(KGlobal.dirs().kfsstnd_prefixes()).split(os.pathsep)
+        prefixes = KGlobal.dirs().kfsstnd_prefixes().split(os.pathsep)
         prefixes = set(prefixes + ['/usr/', '/usr/local/'])
         # if the path is not absolute, add it to all prefixes.
-        for path in conf.readEntry("paths", QVariant(defaultPaths)).toStringList():
-            path = unicode(path)
+        for path in conf.readEntry("paths", defaultPaths):
             if os.path.isabs(path):
                 yield path
             else:
                 for pref in prefixes:
                     yield os.path.join(pref, path)
                 for d in KGlobal.dirs().findDirs("data", path):
-                    yield unicode(d)
+                    yield d
 
     # now find the hyph_xx_XX.dic files
     dicfiles = (f
         for p in paths() if os.path.isdir(p)
-            for f in glob(os.path.join(p, 'hyph_*.dic')) if os.path.isfile(f))
+            for f in glob(os.path.join(p, 'hyph_*.dic')) if os.access(f, os.R_OK))
 
     # default to the users current locale if not used before
     defaultlang = None
@@ -91,7 +91,7 @@ def findDicts():
         for i in lang, lang.split('_')[0]:
             name = KGlobal.locale().languageCodeToName(i)
             if name:
-                name = u'%s (%s)' % (name, lang)
+                name = '{0} ({1})'.format(name, lang)
                 hyphdicts[name] = dic
                 # set current locale as default
                 if lang == language:
@@ -101,8 +101,8 @@ def findDicts():
             hyphdicts[lang] = dic
 
     # if not used before, write the current locale (if existing) as default
-    if defaultlang and unicode(conf.readEntry("lastused", QVariant("")).toString()) not in hyphdicts:
-        conf.writeEntry("lastused", QVariant(defaultlang))
+    if defaultlang and conf.readEntry("lastused", "") not in hyphdicts:
+        conf.writeEntry("lastused", defaultlang)
         conf.sync()
 
 findDicts()
@@ -111,10 +111,19 @@ findDicts()
 def hyphenate(text, mainwindow):
     """
     Ask the user which language to use.
-    Returns None if the user cancels the dialog.
+    Returns None if the user cancels the dialog or no hyphenation pattern files
+    could be found.
     """
+    if not hyphdicts:
+        KMessageBox.sorry(mainwindow, i18n(
+            "Could not find any hyphenation dictionaries.\n\n"
+            "Please install a package containing some and/or or configure the "
+            "search path to find them in the Frescobaldi settings under "
+            "\"Paths.\""))
+        return
+    
     conf = config("hyphenation")
-    lang = conf.readEntry("lastused", QVariant("")).toString()
+    lang = conf.readEntry("lastused", "")
     langs = list(sorted(hyphdicts.keys()))
     index = lang in langs and langs.index(lang) or 0
     
@@ -122,16 +131,17 @@ def hyphenate(text, mainwindow):
     d.setButtons(KDialog.ButtonCode(KDialog.Ok | KDialog.Cancel | KDialog.Help))
     d.setCaption(i18n("Hyphenate Lyrics Text"))
     d.setHelp("lyrics")
-    v = KVBox(d)
-    d.setMainWidget(v)
-    QLabel(i18n("Please select a language:"), v)
-    listbox = QListWidget(v)
+    layout = QVBoxLayout()
+    d.mainWidget().setLayout(layout)
+    layout.addWidget(QLabel(i18n("Please select a language:")))
+    listbox = QListWidget()
+    layout.addWidget(listbox)
     listbox.addItems(langs)
     listbox.setCurrentRow(index)
     listbox.setFocus()
-    if d.exec_() and hyphdicts:
+    if d.exec_():
         lang = langs[listbox.currentRow()]
-        conf.writeEntry("lastused", QVariant(lang))
+        conf.writeEntry("lastused", lang)
         conf.sync()
         # get hyphenator
         h = Hyphenator(hyphdicts[lang])
