@@ -149,7 +149,7 @@ class LilyDoc(QWidget):
         
         # load initial view.
         self.editFontSize = 0
-        styleSheet = KGlobal.dirs().findResource("appdata", "lilydoc.css")
+        styleSheet = KGlobal.dirs().findResource("appdata", "css/lilydoc.css")
         if styleSheet:
             self.view.page().settings().setUserStyleSheetUrl(QUrl(styleSheet))
         self.stack.setCurrentWidget(self.view)
