@@ -22,14 +22,14 @@ from __future__ import unicode_literals
 import os, re, sip, weakref
 from dbus.service import method
 
-from PyQt4.QtCore import QEvent, QSize, QTimer, Qt
+from PyQt4.QtCore import QDir, QEvent, QSize, QTimer, Qt
 from PyQt4.QtGui import (
     QActionGroup, QColor, QIcon, QLabel, QPalette, QPixmap, QProgressBar,
     QStackedWidget, QWidget)
 from PyKDE4.kdecore import KConfig, KGlobal, KUrl, i18n
 from PyKDE4.kdeui import (
     KActionMenu, KApplication, KDialog, KIcon, KIconLoader, KLineEdit, KMenu,
-    KMessageBox, KStandardAction, KStandardGuiItem, KVBox)
+    KMessageBox, KNotification, KStandardAction, KStandardGuiItem, KVBox)
 from PyKDE4.kparts import KParts
 from PyKDE4.ktexteditor import KTextEditor
 
@@ -57,6 +57,9 @@ class MainApp(kateshell.app.MainApp):
         # Put ourselves in environment so ktexteditservice can find us
         os.environ["TEXTEDIT_DBUS_PATH"] = self.serviceName + '/MainApp'
         os.environ["FRESCOBALDI_PID"] = str(os.getpid())
+        # make files for stylesheets and LilyPond-generated pics easily found:
+        QDir.setSearchPaths('css', KGlobal.dirs().findDirs('appdata', 'css/'))
+        QDir.setSearchPaths('pics', KGlobal.dirs().findDirs('appdata', 'pics/'))
     
     def setupConfiguration(self, config):
         # If the application got upgraded, run the install module
@@ -375,6 +378,10 @@ class SymbolManager(object):
 
 class MainWindow(SymbolManager, kateshell.mainwindow.MainWindow):
     """ Our customized Frescobaldi MainWindow """
+    
+    # called when Apply or Ok is clicked in the settings dialog.
+    settingsChanged = Signal()
+    
     def __init__(self, app):
         SymbolManager.__init__(self)
         kateshell.mainwindow.MainWindow.__init__(self, app)
@@ -387,6 +394,7 @@ class MainWindow(SymbolManager, kateshell.mainwindow.MainWindow):
         self.charSelectShortcuts = CharSelectShortcuts(self)
         self.quickInsertShortcuts = QuickInsertShortcuts(self)
         self.jobManager().jobFinished.connect(self.setDocumentStatus)
+        self.jobManager().jobFinished.connect(self.notifyCompileFinished)
         
     @cacheresult
     def actionManager(self):
@@ -442,6 +450,10 @@ class MainWindow(SymbolManager, kateshell.mainwindow.MainWindow):
             import frescobaldi_app.charselect
             return frescobaldi_app.charselect.Dialog(self)
         
+    def createSessionManager(self):
+        """Reimplemented from kateshell.mainwindow.MainWindow"""
+        return SessionManager(self)
+    
     def setupActions(self):
         super(MainWindow, self).setupActions()
         
@@ -698,6 +710,13 @@ class MainWindow(SymbolManager, kateshell.mainwindow.MainWindow):
             import ly.duration
             return ly.duration.makeImplicit(text)
             
+        @self.onSelAction(i18n("Make implicit (per line)"), tooltip=i18n(
+            "Make durations implicit (remove repeated durations), "
+            "except for the first duration in a line."))
+        def durations_implicit_per_line(text):
+            import ly.duration
+            return ly.duration.makeImplicitPerLine(text)
+            
         @self.onSelAction(i18n("Make explicit"),
             tooltip=i18n("Make durations explicit (add duration to every note, "
                          "even if it is the same as the preceding note)."))
@@ -724,30 +743,6 @@ class MainWindow(SymbolManager, kateshell.mainwindow.MainWindow):
             rhythm = KApplication.clipboard().text()
             return ly.duration.applyRhythm(text, rhythm)
         
-        # Bar lines
-        for name, text, title, key in (
-            ("bar_double", "||", i18n("Double bar line"), "Alt+;"),
-            ("bar_end", "|.", i18n("Ending bar line"), "Alt+."),
-            ("bar_dotted", ":", i18n("Dotted bar line"), None),
-            ("bar_dashed", "dashed", i18n("Dashed bar line"), None),
-            ("bar_invisible", "", i18n("Invisible bar line"), None),
-            ("bar_repeat_start", "|:", i18n("Repeat start"), None),
-            ("bar_repeat_double", ":|:", i18n("Repeat both"), None),
-            ("bar_repeat_end", ":|", i18n("Repeat end"), None),
-            ("bar_tick", "'", i18n("Tick bar line"), None),
-            ("bar_single", "|", i18n("Single bar line"), None),
-            ("bar_sws", "|.|", i18n("Small-Wide-Small bar line"), None),
-            ("bar_ws", ".|", i18n("Wide-Small bar line"), None),
-            ("bar_ww", ".|.", i18n("Double wide bar line"), None),
-            ("bar_cswc", ":|.:", i18n("Repeat both (old)"), None),
-            ("bar_cswsc", ":|.|:", i18n("Repeat both (classic)"), None),
-            ("bar_segno", "S", i18n("Segno bar line"), None),
-            ):
-            a = self.act(name, title, key=key, func=(lambda text:
-                lambda: self.currentDocument().manipulator().insertBarLine(
-                    text))(text))
-            self.addSymbol(a, name)
-            
         # Setup lyrics hyphen and de-hyphen action
         @self.onSelAction(i18n("Hyphenate Lyrics Text"), keepSelection=False,
             key="Ctrl+L")
@@ -820,12 +815,12 @@ class MainWindow(SymbolManager, kateshell.mainwindow.MainWindow):
             self.blankStaffPaperWizard().show()
 
     def setupTools(self):
+        QuickInsertTool(self)
         KonsoleTool(self)
         LogTool(self)
-        QuickInsertTool(self)
+        KMidTool(self)
         RumorTool(self)
-        if not config().readEntry("disable pdf preview", False):
-            PDFTool(self)
+        PDFTool(self)
         LilyDocTool(self)
             
     def runLilyPond(self, mode):
@@ -859,11 +854,13 @@ class MainWindow(SymbolManager, kateshell.mainwindow.MainWindow):
                 return # job configure dialog cancelled by user
         else:
             job.preview = mode == "preview"
-            command = None
-            if config("lilypond").readEntry("automatic version", False):
-                docVersion = d.lilyPondVersion()
-                if docVersion:
-                    command = automaticLilyPondCommand(docVersion)
+            # forces the session a certain version?
+            command = self.sessionManager().lilyPondCommand()
+            if not command:
+                if config("lilypond").readEntry("automatic version", False):
+                    docVersion = d.lilyPondVersion()
+                    if docVersion:
+                        command = automaticLilyPondCommand(docVersion)
             job.command = command or lilyPondCommand()
             
         # check if there is not already a job running. We do it now, so the
@@ -946,6 +943,20 @@ class MainWindow(SymbolManager, kateshell.mainwindow.MainWindow):
         self.app.stateManager().cleanup()
         super(MainWindow, self).saveSettings()
 
+    def notifyCompileFinished(self, job, success):
+        """Called when a job exits, notify the user on longer compiles."""
+        if job.buildTime >= 10 and self.isMinimized():
+            n = KNotification("compileFinished", self)
+            if success:
+                n.setText(i18n(
+                    "LilyPond has successfully compiled %1.",
+                    job.document.documentName()))
+            else:
+                n.setText(i18n(
+                    "LilyPond exited with an error compiling %1.",
+                    job.document.documentName()))
+            n.sendEvent()
+
 
 class ApplyRhythmDialog(KDialog):
     def __init__(self, mainwin):
@@ -984,8 +995,11 @@ class KonsoleTool(kateshell.mainwindow.KPartTool):
     _partlibrary = "libkonsolepart"
     _partappname = "Konsole"
     def __init__(self, mainwin):
+        import PyKDE4
+        kde_4_5_or_higher = PyKDE4.kdecore.version() >= 263424
+        icon = "utilities-terminal" if kde_4_5_or_higher else "terminal"
         kateshell.mainwindow.KPartTool.__init__(self, mainwin,
-            "konsole", i18n("Terminal"), "terminal",
+            "konsole", i18n("Terminal"), icon,
             key="Meta+Alt+T", dock=kateshell.mainwindow.Bottom)
         mainwin.currentDocumentChanged.connect(self.sync)
             
@@ -1031,6 +1045,27 @@ class KonsoleTool(kateshell.mainwindow.KPartTool):
         conf.writeEntry("sync", self._sync)
         
 
+class KMidTool(kateshell.mainwindow.Tool):
+    helpAnchor = "kmid"
+    def __init__(self, mainwin):
+        self.failed = False # failed the player to load?
+        kateshell.mainwindow.Tool.__init__(self, mainwin,
+            "kmid", i18n("MIDI Player"), "audio-midi",
+            key="Meta+Alt+M", dock=kateshell.mainwindow.Bottom)
+        
+    def factory(self):
+        import frescobaldi_app.kmid
+        player = frescobaldi_app.kmid.player(self)
+        if player:
+            return player
+        self.failed = True
+        label = QLabel(i18n(
+            "Could not load the KMid part.\n"
+            "Please install KMid 2.4.0 or higher."))
+        label.setAlignment(Qt.AlignCenter)
+        return label
+
+
 class PDFTool(kateshell.mainwindow.KPartTool):
     _partlibrary = "okularpart"
     _partappname = "Okular"
@@ -1046,10 +1081,16 @@ class PDFTool(kateshell.mainwindow.KPartTool):
         self._timer = QTimer()
         self._timer.setSingleShot(True)
         self._timer.timeout.connect(self.timeoutFunc)
-        mainwin.aboutToClose.connect(self._timer.stop)
-        mainwin.currentDocumentChanged.connect(self.sync)
-        mainwin.jobManager().jobFinished.connect(self.openUpdatedPDF)
+        mainwin.aboutToClose.connect(self.appQuit)
+        if self._config["sync"]:
+            mainwin.currentDocumentChanged.connect(self.sync)
+            mainwin.jobManager().jobFinished.connect(self.openUpdatedPDF)
     
+    def appQuit(self):
+        """ Called when the application exits. """
+        self._timer.stop()
+        self.part and self.part.closeUrl()
+        
     def timeoutFunc(self):
         """(Internal) Called when the timer times out.
         
@@ -1085,10 +1126,9 @@ class PDFTool(kateshell.mainwindow.KPartTool):
             self._reallyOpenUrl(self._currentUrl)
             
     def sync(self, doc):
-        if self._config["sync"]:
-            pdfs = doc.updatedFiles()("pdf")
-            if pdfs:
-                self.openUrl(KUrl(pdfs[0]))
+        pdfs = doc.updatedFiles()("pdf")
+        if pdfs:
+            self.openUrl(KUrl(pdfs[0]))
 
     def openUpdatedPDF(self, job):
         pdfs = job.updatedFiles()("pdf")
@@ -1120,13 +1160,21 @@ class PDFTool(kateshell.mainwindow.KPartTool):
                 
     def toggleAction(self, name):
         c = self._config[name] = not self._config[name]
-        if not self.part:
-            return
         # if the part has already loaded, perform these settings.
-        if name == "leftpanel":
+        if name == "leftpanel" and self.part:
             self.part.actionCollection().action("show_leftpanel").setChecked(c)
-        elif name == "minipager":
+        elif name == "minipager" and self.part:
             self._okularMiniBar().setVisible(c)
+        elif name == "sync":
+            if c:
+                self.mainwin.currentDocumentChanged.connect(self.sync)
+                self.mainwin.jobManager().jobFinished.connect(self.openUpdatedPDF)
+                d = self.mainwin.currentDocument()
+                if d:
+                    self.sync(d)
+            else:
+                self.mainwin.currentDocumentChanged.disconnect(self.sync)
+                self.mainwin.jobManager().jobFinished.disconnect(self.openUpdatedPDF)
 
     def _okularMiniBar(self):
         """ get the okular miniBar """
@@ -1179,8 +1227,9 @@ class LogTool(kateshell.mainwindow.Tool):
             key="Meta+Alt+L", dock=kateshell.mainwindow.Bottom,
             widget=QStackedWidget())
         self.logs = {}
-        self.widget.addWidget(QLabel(u"<center>({0})</center>".format(
-            i18n("no log"))))
+        label = QLabel("({0})".format(i18n("no log")))
+        label.setAlignment(Qt.AlignCenter)
+        self.widget.addWidget(label)
         mainwin.currentDocumentChanged.connect(self.showLog)
         mainwin.app.documentClosed.connect(self.removeLog)
         self.widget.destroyed.connect(lambda: self.logs.clear())
@@ -1215,16 +1264,13 @@ class LogTool(kateshell.mainwindow.Tool):
     
     def startJob(self, job):
         log = self.createLog(job.document)
-        log.preview = job.preview
         log.clear()
         job.output.connect(log)
     
     def finishJob(self, job, success):
         log = self.log(job.document)
-        if log:
-            self.mainwin.actionManager().addActionsToLog(job.updatedFiles(), log)
-            if not success:
-                log.show() # even if LP didn't show an error location
+        if log and not success:
+            log.show() # even if LP didn't show an error location
         
     def addMenuActions(self, m):
         def act(name, title):
@@ -1408,7 +1454,18 @@ class QuickInsertShortcuts(kateshell.mainwindow.UserShortcutManager):
     def client(self):
         tool = self.mainwin.tools["quickinsert"]
         tool.materialize()
-        return tool.widget
+        return tool.widget.toolboxWidget
+
+
+class SessionManager(kateshell.mainwindow.SessionManager):
+    def createEditorDialog(self):
+        import frescobaldi_app.sessions
+        return frescobaldi_app.sessions.EditorDialog(self)
+    
+    def lilyPondCommand(self):
+        """Returns the LilyPond command for this session, if configured. """
+        if self.current():
+            return self.config().readEntry('lilypond', '')
 
 
 # Easily get our global config
