@@ -135,7 +135,6 @@ class Titles(QWidget):
         l = QHBoxLayout(self)
         # The html view with the score layout example
         t = QTextBrowser(self)
-        t.setSearchPaths(KGlobal.dirs().findDirs("appdata", "pics"))
         t.setOpenLinks(False)
         t.setOpenExternalLinks(False)
 
@@ -758,7 +757,10 @@ class Builder(object):
         # pitch language:
         language = s.getLanguage()
         if language:
-            self.include("{0}.ly".format(language))
+            if self.lilyPondVersion >= (2, 13, 38):
+                doc.insert(1, ly.dom.Line('\\language "{0}"'.format(language)))
+            else:
+                self.include("{0}.ly".format(language))
 
         # add code blocks, if any:
         for code in self.codeBlocks[::-1]:
@@ -1120,7 +1122,7 @@ a {{
     <td> </td>
     <td align=right>{opus}</td>
 </tr>
-<tr><td colspan=3 align=center><img src='scorewiz.png'></td></tr>
+<tr><td colspan=3 align=center><img src='pics:scorewiz.png'></td></tr>
 <tr><td colspan=3 align=center>{copyright} <i>({copyrightmsg})</i></td></tr>
 <tr><td colspan=3 align=center>{tagline} <i>({taglinemsg})</i></td></tr>
 </table></body></html>
