@@ -101,6 +101,7 @@ class SettingsDialog(KPageDialog):
             page.saveSettings()
             page.applySettings()
         self.changed(False)
+        self.mainwin.settingsChanged()
             
     def slotCurrentPageChanged(self, current, before):
         self.setHelp(getattr(current.widget(), "help", "settings-dialog"))
@@ -305,10 +306,6 @@ class RunningLilyPond(CheckGroup):
             self.addCheckBox(i18n("Run LilyPond with verbose output"),
                 "verbose lilypond output", False))
 
-        layout.addWidget(
-            self.addCheckBox(i18n("Disable the built-in PDF preview"),
-                "disable pdf preview", False))
-        
         h = KHBox()
         QLabel(i18n("LilyPond include path:"), h)
         self.includePath = FilePathEdit(h)
@@ -329,17 +326,6 @@ class RunningLilyPond(CheckGroup):
         conf = config("preferences")
         conf.writePathEntry("lilypond include path",
             self.includePath.value())
-    
-    def applySettings(self):
-        # disable or enable the builtin PDF preview
-        mainwin = self.page.dialog.mainwin
-        disable = config("preferences").readEntry("disable pdf preview", False)
-        running = "pdf" in mainwin.tools
-        if disable and running:
-            mainwin.tools["pdf"].delete()
-        elif not disable and not running:
-            tool = frescobaldi_app.mainapp.PDFTool(mainwin)
-            tool.sync(mainwin.currentDocument())
 
 
 class SavingDocument(CheckGroup):
@@ -678,7 +664,6 @@ class LilyPondVersions(SettingsGroup):
         self.instances.addItem(info)
         self.instances.setCurrentItem(info)
         self.auto.setChecked(False)
-        info.changed()
         
     def loadSettings(self):
         self.instances.clear()
@@ -690,9 +675,9 @@ class LilyPondVersions(SettingsGroup):
         paths.sort(key=lambda path: ly.version.LilyPondInstance(path).version() or (999,))
         for path in paths:
             info = LilyPondInfoItem(path)
+            info.loadSettings(conf.group(path))
             info.default = path == default
             self.instances.addItem(info)
-            info.loadSettings(conf.group(path))
             if info.default:
                 self.instances.setCurrentItem(info)
     
@@ -753,9 +738,15 @@ class LilyPondInfoList(ListEdit):
         dlg.loadInfo(item)
         if dlg.exec_():
             dlg.saveInfo(item)
+            # force reread of LilyPond version
+            ly.version.LilyPondInstance(item.lilypond, False)
             return True
         return False
         
+    def itemChanged(self, item):
+        item.changed()
+        self.setCurrentItem(item)
+
 
 class LilyPondInfo(object):
     """
@@ -787,15 +778,10 @@ class LilyPondInfo(object):
             ("lilypond-book", i18n("Lilypond-book:")),
         )
 
-    def changed(self):
-        """ Implement to be notified of changes. """
-        pass
-
     def loadSettings(self, group):
         for cmd, descr in self.commandNames():
             self.commands[cmd] = group.readEntry(cmd, cmd)
         self.auto = group.readEntry("auto", True)
-        self.changed()
     
     def saveSettings(self, group):
         for cmd, descr in self.commandNames():
@@ -827,7 +813,6 @@ class LilyPondInfoItem(QListWidgetItem, LilyPondInfo):
             self.setToolTip(i18n("Can't determine LilyPond version."))
         if self.default:
             lp += " [{0}]".format(i18n("default"))
-            
             # reset the default state for others
             for c in range(self.listWidget().count()):
                 item = self.listWidget().item(c)
@@ -844,9 +829,14 @@ class LilyPondInfoDialog(KDialog):
     def __init__(self, parent):
         KDialog.__init__(self, parent)
         self.setButtons(KDialog.ButtonCode(
-            KDialog.Ok | KDialog.Cancel | KDialog.Help))
+            KDialog.Ok | KDialog.Cancel | KDialog.Help | KDialog.User1))
         self.setCaption(i18n("LilyPond"))
         self.setHelp("settings-paths-lilypond")
+        self.setButtonText(KDialog.User1, i18n("Download..."))
+        self.setButtonIcon(KDialog.User1, KIcon("download"))
+        self.setButtonToolTip(KDialog.User1, i18n(
+            "Download new binary LilyPond releases."))
+        self.user1Clicked.connect(self.downloadLilyPond)
         layout = QGridLayout(self.mainWidget())
         
         l = QLabel(i18n("LilyPond Command:"))
@@ -887,7 +877,10 @@ class LilyPondInfoDialog(KDialog):
             info.commands[name] = widget.text()
         info.default = self.default.isChecked()
         info.auto = self.auto.isChecked()
-        info.changed()
+    
+    def downloadLilyPond(self):
+        from frescobaldi_app.download import LilyPondDownloadDialog
+        LilyPondDownloadDialog(self).exec_()
 
 
 class SessionsStartup(SettingsGroup):
