@@ -28,7 +28,7 @@ from dbus.service import method, signal
 
 from signals import Signal
 
-from PyQt4.QtCore import QObject, Qt, SIGNAL
+from PyQt4.QtCore import QObject, QThread, Qt, SIGNAL
 from PyQt4.QtGui import QCursor
 from PyKDE4.kdecore import i18n, KConfig, KGlobal, KUrl
 from PyKDE4.kdeui import KApplication, KMessageBox, KStandardGuiItem
@@ -130,18 +130,12 @@ class MainApp(DBusItem):
     documentMaterialized = Signal()
     documentClosed = Signal()
     
-    excepthook = Signal()
-    
     iface = DBUS_IFACE_PREFIX + "MainApp"
     defaultEncoding = 'UTF-8'
     defaultMode = None
     fileTypes = []
     
     def __init__(self, servicePrefix):
-        # make it easy to connect more error handlers
-        self.excepthook.connect(sys.excepthook)
-        sys.excepthook = self.excepthook
-        
         # We manage our own documents.
         self.documents = []
         self.history = []       # latest shown documents
@@ -256,7 +250,7 @@ class MainApp(DBusItem):
                     self.mainwin.sessionManager().switch(session)
         if len(self.documents) == 0:
             self.createDocument().setActive()
-        self.excepthook.connect(self.showException)
+        sys.excepthook = self.handleException
         self.mainwin.show()
         self.kapp.exec_()
         KGlobal.config().sync()
@@ -310,7 +304,7 @@ class MainApp(DBusItem):
         return self.mainwin.sessionManager().switch(session)
         
     @method(iface, in_signature='', out_signature='s')
-    def startSession(self, session):
+    def currentSession(self):
         """Returns the name of the current session.
         
         Returns "none" if there is no active session.
@@ -351,10 +345,19 @@ class MainApp(DBusItem):
         """Returns the version of our application."""
         return KGlobal.mainComponent().aboutData().version()
 
-    def showException(self, exctype, excvalue, exctb):
-        """Called when a Python exception goes unhandled."""
-        import kateshell.exception
-        kateshell.exception.showException(self, exctype, excvalue, exctb)
+    def handleException(self, exctype, excvalue, exctb):
+        """Called when a Python exception goes unhandled.
+        
+        Catches KeyboardInterrupt and shows a dialog for other errors.
+        Also writes the traceback to stderr.
+        
+        """
+        from traceback import format_exception
+        sys.stderr.write(''.join(format_exception(exctype, excvalue, exctb)))
+        
+        if exctype != KeyboardInterrupt:
+            from kateshell.exception import ExceptionDialog
+            ExceptionDialog(self, exctype, excvalue, exctb)
 
     def keepMetaInfo(self):
         """Returns whether meta information about documents should be kept.
@@ -383,6 +386,12 @@ class MainApp(DBusItem):
             yield
         finally:
             KApplication.restoreOverrideCursor()
+    
+    def debug(self, message):
+        """Write message to stderr."""
+        message = "{0}: {1}\n".format(
+            self.programName(), "\n ".join(message.splitlines()))
+        sys.stderr.write(message)
 
 
 class Document(DBusItem):
@@ -1092,15 +1101,63 @@ class CursorTranslator(object):
             column = resolvetabs_indices(column, self.savedTabs[line])
         cursor = KTextEditor.Cursor(line, column)
         if self.iface:
-            self.iface.smartMutex().lock()
-            self.iface.useRevision(self.revision)
-            cursor = self.iface.translateFromRevision(cursor,
-                KTextEditor.SmartCursor.MoveOnInsert)
-            self.iface.clearRevision()
-            self.iface.smartMutex().unlock()
+            # Just because KDE 4.5 does a qFatal if useRevision is called in the
+            # main thread, and the Python KDE4 bindings not yet provide the new
+            # MovingInterface stuff, we need to create a background thread just
+            # to translate cursors from a certain document revision.
+            @anonymousThread
+            def translateCursor(cursor):
+                self.iface.smartMutex().lock()
+                self.iface.useRevision(self.revision)
+                cursor = self.iface.translateFromRevision(cursor,
+                    KTextEditor.SmartCursor.MoveOnInsert)
+                self.iface.clearRevision()
+                self.iface.smartMutex().unlock()
+                return cursor
+            cursor = translateCursor(cursor)
         return cursor
 
 
+class _AnonymousThread(QThread):
+    """Runs a function in an anonymous QThread.
+    
+    Exceptions are re-raised in the main thread.
+    
+    """
+    def __init__(self, func, args, kwargs):
+        super(_AnonymousThread, self).__init__()
+        self.func = lambda: func(*args, **kwargs)
+        self._result = None
+        self._exc_info = None
+        self.start()
+        self.wait()
+    
+    def run(self):
+        try:
+            self._result = self.func()
+        except:
+            self._exc_info = sys.exc_info()
+            sys.exc_clear()
+    
+    def result(self):
+        """Returns a tuple (result, exception)."""
+        return self._result, self._exc_info
+
+
+def anonymousThread(func):
+    """Returns a wrapper for a func to run it in an anonymous QThread.
+    
+    When called, waits for the function to complete and returns its result.
+    
+    """
+    def wrapper(*args, **kwargs):
+        result, exc_info = _AnonymousThread(func, args, kwargs).result()
+        if exc_info:
+            raise exc_info[1], None, exc_info[2]
+        return result
+    return wrapper
+
+
 def tabindices(text):
     """Returns a list of positions in text at which a tab character is found.
     
