@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
+from __future__ import unicode_literals
+
 shorthands = {
     'marcato': '^',
     'stopped': '+',
@@ -68,6 +70,7 @@ def groups(i18nFunc=None):
         (i18n("Other"), (
             ('upbow', i18n("Upbow")),
             ('downbow', i18n("Downbow")),
+            ('snappizzicato', i18n("Snappizzicato")),
             ('open', i18n("Open (e.g. brass)")),
             ('stopped', i18n("Stopped (e.g. brass)")),
             ('flageolet', i18n("Flageolet")),
@@ -76,5 +79,16 @@ def groups(i18nFunc=None):
             ('rheel', i18n("Right heel")),
             ('ltoe', i18n("Left toe")),
             ('rtoe', i18n("Right toe")),
+            ('halfopen', i18n("Half open (e.g. hi-hat)")),
             )),
     )
+
+def articulations(i18nFunc=None):
+    """
+    Yields two-tuples (name, translated title) for all articulations,
+    usable to e.g. create a dict.
+    """
+    for title, group in groups(i18nFunc):
+        for articulation in group:
+            yield articulation
+
