@@ -35,6 +35,7 @@ keywords = (
     #'grobdescriptions',
     'include',
     #'invalid',
+    'language',
     'name',
     #'objectid',
     'once',
@@ -74,6 +75,7 @@ musiccommands = (
     'afterGrace',
     'afterGraceFraction',
     'aikenHeads',
+    'aikenHeadsMinor',
     'allowPageTurn',
     'alternative',
     'AncientRemoveEmptyStaffContext',
@@ -192,6 +194,7 @@ musiccommands = (
     'ff',
     'fff',
     'ffff',
+    'fffff',
     'figuremode',
     'figures',
     'finalis',
@@ -201,6 +204,8 @@ musiccommands = (
     'fp',
     'frenchChords',
     'fullJazzExceptions',
+    'funkHeads',
+    'funkHeadsMinor',
     'fz',
     'germanChords',
     'glissando',
@@ -335,6 +340,7 @@ musiccommands = (
     'rightHandFinger',
     'rtoe',
     'sacredHarpHeads',
+    'sacredHarpHeadsMinor',
     'scaleDurations',
     'scoreTweak',
     'segno',
@@ -365,6 +371,8 @@ musiccommands = (
     'snappizzicato',
     'sostenutoOff',
     'sostenutoOn',
+    'southernHarmonyHeads',
+    'southernHarmonyHeadsMinor',
     'sp',
     'spacingTweaks',
     'spp',
@@ -392,6 +400,7 @@ musiccommands = (
     'stropha',
     'sustainOff',
     'sustainOn',
+    'tabFullNotation',
     'tag',
     'teeny',
     'tempo',
@@ -446,6 +455,8 @@ musiccommands = (
     'voiceThreeStyle',
     'voiceTwo',
     'voiceTwoStyle',
+    'walkerHeads',
+    'walkerHeadsMinor',
     'whiteTriangleMarkup',
     'withMusicProperty',
 )
@@ -530,6 +541,7 @@ markupcommands_nargs = (
     'large',
     'larger',
     'left-align',
+    'left-brace',
     'left-column',
     'line',
     'lookup',
@@ -544,6 +556,7 @@ markupcommands_nargs = (
     'number',
     'postscript',
     'right-align',
+    'right-brace',
     'right-column',
     'roman',
     'rounded-box',
@@ -590,6 +603,7 @@ markupcommands_nargs = (
     'pad-around',
     'pad-markup',
     'pad-x',
+    'path',     # added in LP 2.13.31
     'raise',
     'rotate',
     'translate',
@@ -654,6 +668,7 @@ contexts = (
     'StaffGroup',
     'TabStaff',
     'TabVoice',
+    'Timing',
     'VaticanaStaff',
     'VaticanaVoice',
     'Voice',
@@ -1518,9 +1533,12 @@ contextproperties = (
     'barNumberVisibility',
     'bassFigureFormatFunction',
     'bassStaffProperties',
+    'baseMoment',
+    'beamExceptions',
     'beamSettings',
     'beatGrouping',
     'beatLength',
+    'beatStructure',
     'chordChanges',
     'chordNameExceptions',
     'chordNameExceptionsFull',
